// $Id: request.C,v 1.13 2003/09/18 13:02:25 dvermeir Exp $
#include "ticket.h"
#include "request.h"
#include "server.h"
#include <dvnet/inetaddress.h>

Dv::Ticket::Request::Ref
Dv::Ticket::Request::make(const Dv::Xml::Document& req,
  Dv::Util::ref<Dv::Ssl::SslSocket> client, Dv::Ticket::Server&
  server) {
Dv::Xml::Node::Ref r(req.root());
if (r/"login") 
  return Ref(new LoginRequest(req, client, server));
else if (r/"anon-login") 
  return Ref(new AnonLoginRequest(req, client, server));
else if (r/"validate") 
  return Ref(new ValidateRequest(req, client, server));
return Ref(0);
}

Dv::Ticket::Request::Request(const Dv::Xml::Document& req,
  Dv::Util::ref<Dv::Ssl::SslSocket> client, Dv::Ticket::Server& server):
    req_(req), root_(req.root()), client_(client),
    host_(Net::InetAddress::by_name(client->host())->dot_address()),
    server_(server) {
// log << "Dv::Ticket::Request::Request: client = " << client->host() << std::endl;
}

Dv::Xml::Document&
Dv::Ticket::LoginRequest::handle(Dv::Xml::Document& result) {
try {
  Dv::Xml::Node::Ref top = root()/"login";
  std::string name = top/"user"/"name";
  std::string passwd = top/"user"/"passwd";
  std::string info = top/"user"/"info";
  size_t duration = top/"duration";
  std::string client_host(host());
  // if (host() == "127.0.0.1") {
    Dv::Xml::Node::Ref host_ref(top/"host");
    if (host_ref)
      client_host = host_ref.str();
  //  }
  Dv::Ticket::User user(server().authenticator().validate(client_host, name, passwd, info));
  Dv::Ticket::Ticket::Ref ticket(
    server().make_ticket(user, client_host, duration)
    );
  result = ticket->xml();
  return result;
  }
catch (std::exception& e) {
  server().log() << "LoginRequest::handle: " << e.what() << std::endl;
  return result;
  }
}

Dv::Xml::Document&
Dv::Ticket::AnonLoginRequest::handle(Dv::Xml::Document& result) {
try {
  Dv::Xml::Node::Ref top = root()/"anon-login";
  std::string name = top/"anon-user"/"name";
  std::string info = top/"anon-user"/"info";
  size_t duration = top/"duration";
  Dv::Ticket::User user(server().authenticator().anon_validate(host(), name, info));
  Dv::Ticket::Ticket::Ref ticket(
    server().make_ticket(user, host(), duration)
    );
  result = ticket->xml();
  return result;
  }
catch (std::exception& e) {
  server().log() << "LoginRequest::handle: " << e.what() << std::endl;
  return result;
  }
}

Dv::Xml::Document&
Dv::Ticket::ValidateRequest::handle(Dv::Xml::Document& result) {
try {
  Dv::Xml::Node::Ref ticket_node(root()/"validate"/"ticket");
  Dv::Ticket::Ticket::SERIAL serial = ticket_node/"serial";

  std::string owner(host());
  Dv::Xml::Node::Ref host_node(ticket_node/"host");
  server().log() << "ValidateRequest: host_node = " << host_node << std::endl;
  if (host_node)
    owner = host_node.str();
  server().log() << "ValidateRequest: owner = " << owner << std::endl;
  Dv::Ticket::Ticket::Ref ticket(server().validate_ticket(serial, owner));
  result = ticket->xml();
  return result;
  }
catch (std::exception& e) {
  server().log() << "ValidateRequest::handle: " << e.what() << std::endl;
  return result;
  }
}

