#ifndef DV_TICKET_TRUSTED_IP_COLLECTION
#define DV_TICKET_TRUSTED_IP_COLLECTION
// $Id: test-trusted-ip-collection.h,v 1.1 2003/07/23 08:38:13 dvermeir Exp $
#include <stdexcept>
#include <list>

namespace Dv {
namespace Ticket {
/**
 * a TrustedIpCollecion is a thin wrapper around a collection
 * of strings (dotaddresses of hosts) which is read from
 * and saved to a file. It can be used in an Dv::Ticket::Authenticator
 * implementation e.g. to decide whether a client is allowed to
 * obtain an anonymous ticket (e.g. only if the client calls
 * from a host in the collection).
 *
 * @see Dv::Ticket::Request Dv::Ticket::Authenticator
 */
class TrustedIpCollection {
public:
  /** Type of container in which dot address prefixes are kept. */
  typedef std::list<std::string> Container;

  /** Constructor.
   * @param fn name of file containing trusted ip info
   *  The file contains one (prefix of) a numeric IP address
   *  per line. Lines beginning with '#' are ignored.
   * @exception std::runtime_error if anything goes wrong.
   */
  TrustedIpCollection(const std::string& fn) throw (std::runtime_error);
  /** Destructor. */
  ~TrustedIpCollection() {}
 
  /** Is this IP address trusted?
   * @return true iff the one of the strings in the collection
   *   is a prefix of the (dot address) argument.
   * @warning does not check whether the argument is a valid
   *   dot address
   */
  bool trusted(const std::string& address) const throw ();
private:
  Container container_;
};

}}
#endif
