// $Id: ticket.C,v 1.14 2003/10/21 12:37:52 dvermeir Exp $

#include <dvutil/random.h>
#include <dvutil/ntostr.h>
#include <dvnet/inetaddress.h>
#include <dvssl/sslsocket.h>
#include <dvssl/x509cert.h>

#include "xml-sndrcv.h"
#include  "ticket.h"

//////////// Buy a new ticket from host:port.
Dv::Util::ref<Dv::Ticket::Ticket>
Dv::Ticket::Ticket::buy(const std::string& server_host, int server_port, const std::string& name, 
  const std::string& password, const std::string& info, size_t duration)
    throw () {
try {
  Dv::Ssl::SslContextV23 context;

  Dv::Ssl::SslSocket so(context, server_host, server_port);

  if (!so)
    throw std::runtime_error(so.strerror());

  Dv::Ssl::X509Certificate login_cert(so);
  // std::cout << "login_certificate name = " << login_cert.name() << std::endl
  //     << "login_certificate issuer = " << login_cert.issuer() << std::endl;

  Dv::Xml::Node request("dvticket-request");
  Dv::Xml::Node::Ref login(request >> "login");
  Dv::Xml::Node::Ref user_xml(login >> "user");
  (user_xml >> "name") = name;
  (user_xml >> "passwd") = Dv::Xml::to_html(password);
  (user_xml >> "info") = info;
  (login >> "duration") = duration;
  std::cerr << request << std::endl;
  send(Dv::Xml::Document(request), so);
  Dv::Xml::Document reply;
  receive(reply, so);
  Dv::Xml::Node::Ref root(reply.root());

  if (root.name() == "ticket") 
    return Ref(new Ticket(root, server_host, server_port));
  
  throw std::runtime_error("request refused");
  }
catch (std::exception& e) {
  return Ref();
  }
}

Dv::Util::ref<Dv::Ticket::Ticket>
Dv::Ticket::Ticket::buy(const std::string& client_host,
  const std::string& server_host, int server_port, const std::string& name, 
  const std::string& password, const std::string& info, size_t duration)
    throw () {
try {
  Dv::Ssl::SslContextV23 context;

  Dv::Ssl::SslSocket so(context, server_host, server_port);

  if (!so)
    throw std::runtime_error(so.strerror());

  Dv::Ssl::X509Certificate login_cert(so);
  // std::cout << "login_certificate name = " << login_cert.name() << std::endl
  //    << "login_certificate issuer = " << login_cert.issuer() << std::endl;

  std::string client_dot_address = Dv::Net::InetAddress::by_name(client_host)->dot_address();

  Dv::Xml::Node request("dvticket-request");
  Dv::Xml::Node::Ref login(request >> "login");
  Dv::Xml::Node::Ref user_xml(login >> "user");
  (user_xml >> "name") = name;
  (user_xml >> "passwd") = Dv::Xml::to_html(password);
  (user_xml >> "info") = info;
  (login >> "duration") = duration;
  (login >> "host") = client_dot_address;

  send(Dv::Xml::Document(request), so);
  Dv::Xml::Document reply;
  receive(reply, so);
  Dv::Xml::Node::Ref root(reply.root());

  if (root.name() == "ticket") 
    return Ref(new Ticket(root, server_host, server_port));
  
  throw std::runtime_error("request refused");
  }
catch (std::exception& e) {
  return Ref();
  }
}
//////////// Buy an anonymous ticket from host:port.
Dv::Util::ref<Dv::Ticket::Ticket>
Dv::Ticket::Ticket::buy_anon(const std::string& server_host, int server_port, const std::string& name, 
  const std::string& info, size_t duration) throw () {
try {
  Dv::Ssl::SslContextV23 context;

  Dv::Ssl::SslSocket so(context, server_host, server_port);

  if (!so)
    throw std::runtime_error(so.strerror());

  Dv::Ssl::X509Certificate login_cert(so);
  // std::cout << "login_certificate name = " << login_cert.name() << std::endl
  //    << "login_certificate issuer = " << login_cert.issuer() << std::endl;

  Dv::Xml::Node request("dvticket-request");
  Dv::Xml::Node::Ref login(request >> "anon-login");
  Dv::Xml::Node::Ref user_xml(login >> "anon-user");
  (user_xml >> "name") = name;
  (user_xml >> "info") = info;
  (login >> "duration") = duration;

  send(Dv::Xml::Document(request), so);
  Dv::Xml::Document reply;
  receive(reply, so);
  Dv::Xml::Node::Ref root(reply.root());

  if (root.name() == "ticket") 
    return Ref(new Ticket(root, server_host, server_port));
  
  throw std::runtime_error("request refused");
  }
catch (std::exception& e) {
  return Ref();
  }
}

//////////// Validate a ticket with server on host:port.
Dv::Ticket::Ticket::Ref 
Dv::Ticket::Ticket::validate(const std::string& server_host, int server_port, SERIAL serial) 
  throw () {
try {
  Dv::Ssl::SslContextV23 context;

  Dv::Ssl::SslSocket so(context, server_host, server_port);

  if (!so)
    throw std::runtime_error(so.strerror());

  Dv::Ssl::X509Certificate login_cert(so);
  // std::cout << "login_certificate name = " << login_cert.name() << std::endl
  //   << "login_certificate issuer = " << login_cert.issuer() << std::endl;

  Dv::Xml::Node request("dvticket-request");
  (request >> "validate" >> "ticket" >> "serial") = serial;

  send(Dv::Xml::Document(request), so);
  Dv::Xml::Document reply;
  receive(reply, so);
  Dv::Xml::Node::Ref root(reply.root());

  if (root.name() == "ticket") 
    return Ref(new Ticket(root, server_host, server_port));
    
  throw std::runtime_error("request refused");
  }
catch (std::exception& e) {
  return Ref();
  }
}

Dv::Ticket::Ticket::Ref 
Dv::Ticket::Ticket::validate( const std::string& server_host, int server_port,
  const std::string& host, SERIAL serial) throw () {
try {
  Dv::Ssl::SslContextV23 context;

  Dv::Ssl::SslSocket so(context, server_host, server_port);

  if (!so)
    throw std::runtime_error(so.strerror());

  Dv::Ssl::X509Certificate login_cert(so);
  // std::cout << "login_certificate name = " << login_cert.name() << std::endl
  //   << "login_certificate issuer = " << login_cert.issuer() << std::endl;

  Dv::Xml::Node request("dvticket-request");
  Dv::Xml::Node::Ref ticket_node(request >> "validate" >> "ticket");
  (ticket_node >> "serial") = serial;
  (ticket_node >> "host") = host;

  send(Dv::Xml::Document(request), so);
  Dv::Xml::Document reply;
  receive(reply, so);
  Dv::Xml::Node::Ref root(reply.root());

  if (root.name() == "ticket") {
    SERIAL serial(root/"serial");
    Dv::Util::Date expiration_date((root/"expires").str());
    Dv::Ticket::User user(root/"user");
    return Ref(new Ticket(user, serial, expiration_date, server_host, server_port, host));
    }
  throw std::runtime_error("request refused");
  }
catch (std::exception& e) {
  return Ref();
  }
}

Dv::Xml::Node
Dv::Ticket::Ticket::xml(const std::string& node_name) const {
Dv::Xml::Node node(node_name);
(node >> "serial") = Dv::Util::tostring(serial());
node >> user().xml();
(node >> "expires") = expiration_date_.str("%Y-%m-%d %H:%M:%S");
Dv::Xml::Node::Ref server(node >> "server");
(server >> "host") = server_host();
(server >> "port") = server_port();
(node >> "owner") = owner_host();
return node;
}

Dv::Ticket::Ticket::Ticket(const Dv::Xml::Node::Ref& root, const std::string& server_host, int server_port):
  server_host_(server_host), server_port_(server_port) {
serial_ = (root/"serial");
expiration_date_ = Dv::Util::Date((root/"expires").str());
user_ = User(root/"user");
owner_host_ = (root/"owner").str();
}
